Localize.FormatError exception (Localize v0.6.0)

Copy Markdown View Source

Exception raised when a value cannot be formatted by the requested message format function due to an incompatible type or missing configuration.