Localize.LocaleNotFoundInCacheError exception (Localize v0.38.0)

Copy Markdown View Source

Exception raised when a locale file is not found in the locale cache directory, or cannot be read for another I/O reason.

Summary

Types

t()

@type t() :: %Localize.LocaleNotFoundInCacheError{
  __exception__: term(),
  locale_id: atom(),
  path: Path.t() | nil,
  posix_error: :file.posix() | term() | nil
}