Exception raised when two date or datetime values are considered
equal at every field used by Localize.Interval.greatest_difference/2,
so no greater-than-zero interval field can be returned.
Exception raised when two date or datetime values are considered
equal at every field used by Localize.Interval.greatest_difference/2,
so no greater-than-zero interval field can be returned.