Exception raised when a value cannot be formatted by the requested message format function due to an incompatible type or missing configuration.
Exception raised when a value cannot be formatted by the requested message format function due to an incompatible type or missing configuration.