Exception raised when a locale file is not found in the locale cache directory, or cannot be read for another I/O reason.
Summary
Types
@type t() :: %Localize.LocaleNotFoundInCacheError{ __exception__: term(), locale_id: atom(), path: Path.t() | nil, posix_error: :file.posix() | term() | nil }