Exception raised when a unit conversion cannot be performed.
This may occur because the units are not convertible, the unit requires a special conversion that is not supported, or the conversion involves mixed units.
Exception raised when a unit conversion cannot be performed.
This may occur because the units are not convertible, the unit requires a special conversion that is not supported, or the conversion involves mixed units.